@@ -24,7 +24,7 @@ else:
                     return None
             self.fail('invalid', input=data)
 
-        def to_representation(self, value, row_data):
+        def to_representation(self, value, row_data=None):
             parent_value = super().to_representation(value, row_data)
             if value in self.NULL_VALUES and self.allow_null:
                 return None
