@@ -2,6 +2,10 @@ import json as jsonlib
 from html import unescape
 
 from django import template
+from django.template.base import FilterExpression, kwarg_re, NodeList
+from django.template.defaulttags import IfNode
+from django.template.loader import get_template
+from django.template.loader_tags import BlockNode, ExtendsNode
 from django.utils.safestring import mark_safe
 from django.utils.translation import ugettext_lazy as _
 from rest_framework.templatetags import rest_framework as drftt
@@ -35,11 +39,6 @@ def items(value):
     return drftt.items(value)
 
 
-@register.filter
-def add_nested_class(value):
-    return drftt.add_nested_class(value)
-
-
 @register.filter
 def format_value(value):
     # TODO: this one pending removal on tasks #62 in #63 completion
@@ -296,3 +295,163 @@ def startswith(text, starts):
 @register.filter
 def handle_rtf_linebreaks(value):
     return mark_safe(unescape(value))
+
+
+def parse_tag(token, parser):
+    """
+    Generic template tag parser.
+
+    Returns a three-tuple: (tag_name, args, kwargs)
+
+    tag_name is a string, the name of the tag.
+
+    args is a list of FilterExpressions, from all the arguments that didn't look like kwargs,
+    in the order they occurred, including any that were mingled amongst kwargs.
+
+    kwargs is a dictionary mapping kwarg names to FilterExpressions, for all the arguments that
+    looked like kwargs, including any that were mingled amongst args.
+
+    (At rendering time, a FilterExpression f can be evaluated by calling f.resolve(context).)
+    """
+    # Split the tag content into words, respecting quoted strings.
+    bits = token.split_contents()
+
+    # Pull out the tag name.
+    tag_name = bits.pop(0)
+
+    # Parse the rest of the args, and build FilterExpressions from them so that
+    # we can evaluate them later.
+    args = []
+    kwargs = {}
+    for bit in bits:
+        # Is this a kwarg or an arg?
+        match = kwarg_re.match(bit)
+        kwarg_format = match and match.group(1)
+        if kwarg_format:
+            key, value = match.groups()
+            kwargs[key] = FilterExpression(value, parser)
+        else:
+            args.append(FilterExpression(bit, parser))
+
+    return tag_name, args, kwargs
+
+
+class ExtendTemplateNode(template.Node):
+    """
+    Node for rendering extended template
+    """
+
+    def __init__(self, nodelist, template_name, kwargs, only, multiline):
+        self.nodelist = nodelist
+        self.template_name = template_name
+        self.kwargs = kwargs
+        self.only = only
+        self.multiline = multiline
+
+    def rearrange_blocks(self, nodelist, blocks):
+        for idx, node in enumerate(nodelist):
+            if hasattr(node, 'nodelist'):
+                if isinstance(node, BlockNode):
+                    if node.name in blocks:
+                        nodelist[idx] = blocks[node.name]
+                if isinstance(node, IfNode):
+                    for nodelist_tmp in nodelist[idx].conditions_nodelists:
+                        for item_id, item in enumerate(nodelist_tmp):
+                            if isinstance(item, NodeList):
+                                self.rearrange_blocks(nodelist_tmp[item_id], blocks)
+                else:
+                    self.rearrange_blocks(nodelist[idx].nodelist, blocks)
+        return nodelist
+
+    def get_all_blocks(self, nodelist):
+        blocks = []
+        for node in [node for node in nodelist if isinstance(node, BlockNode)]:
+            blocks.extend(node)
+            blocks.extend(self.get_all_blocks(node.nodelist))
+        return blocks
+
+    @staticmethod
+    def get_node_list(nodelist):
+        while len(nodelist) == 1 and isinstance(nodelist[0], ExtendsNode):
+            nodelist = nodelist[0].nodelist
+        return nodelist
+
+    def render(self, context):
+
+        kwargs = {key: value.resolve(context) for key, value in self.kwargs.items()}
+        try:
+            if 'template_name_var' in kwargs:
+                extending_template = get_template(kwargs.get('template_name_var'))
+            else:
+                extending_template = get_template(self.template_name.resolve())
+        except:
+            return ''
+
+        if 'block' in kwargs:
+            nodelist = self.get_node_list(extending_template.template.nodelist)
+            block_nodes = self.get_all_blocks(nodelist)
+            extending_template.template.nodelist = NodeList(
+                [node for node in block_nodes if node.name == kwargs.get('block')]
+            )
+            extending_template.template.nodelist.contains_nontext = True
+
+        if self.multiline:
+            blocks = {node.name: node for node in self.nodelist if isinstance(node, BlockNode)}
+            nodelist = self.get_node_list(extending_template.template.nodelist)
+
+            self.rearrange_blocks(nodelist, blocks)
+
+        if self.only:
+            flattened_context = kwargs
+        else:
+            flattened_context = context.flatten()
+            flattened_context.update(kwargs)
+
+        content = extending_template.render(flattened_context)
+        content = mark_safe(content)
+        return content
+
+
+def get_extend_data(token, parser):
+    tag_name, args, kwargs = parse_tag(token, parser)
+
+    template_name = None
+    if 'template_name_var' not in kwargs:
+        template_name = args[0]
+
+    return args, kwargs, template_name
+
+
+@register.tag("extendtemplate")
+def do_extendtemplate(parser, token):
+    """
+    Similar to base tag include, but with possibility to use blocks
+    This tag is used as single line (without closing tag) For multiline use extendtemplateblock
+
+    Template name can be provided in two ways:
+      - If template name is static than declare it in first arg
+      - If template name is in variable than declare variable name in 'template_name_var' kwarg
+
+    Keyword parameters will be used as context when rendering template
+    If there is "only" in parameters only keyword parameters will be used as context for rendering.
+    Otherwise keywords will be added to existing context
+    """
+
+    args, kwargs, template_name = get_extend_data(token, parser)
+
+    return ExtendTemplateNode([], template_name, kwargs, 'only' in (x.token for x in args), False)
+
+
+@register.tag("extendtemplateblock")
+def do_extendtemplateblock(parser, token):
+    """
+    Same do_extendtemplate, only multiline... with posibillity to use blocks
+    """
+
+    args, kwargs, template_name = get_extend_data(token, parser)
+
+    # This si multiline block with closing tag
+    nodelist = parser.parse(('endextendtemplateblock',))
+    parser.delete_first_token()
+
+    return ExtendTemplateNode(nodelist, template_name, kwargs, 'only' in (x.token for x in args), True)
